@@ -4,7 +4,7 @@
  * See the IPFILTER.LICENCE file for details on licencing.
  *
  * @(#)ip_nat.h	1.5 2/4/96
- * $Id: ip_nat.h,v 2.90.2.11 2005/06/18 02:41:32 darrenr Exp $
+ * $Id: ip_nat.h,v 2.90.2.14 2006/10/28 06:56:03 darrenr Exp $
  */
 
 #ifndef	__IP_NAT_H__
@@ -121,6 +121,7 @@ typedef	struct	nat	{
 	int		nat_hv[2];
 	char		nat_ifnames[2][LIFNAMSIZ];
 	int		nat_rev;		/* 0 = forward, 1 = reverse */
+	int		nat_redir;		/* copy of in_redir */
 } nat_t;
 
 #define	nat_inip	nat_inip6.in4
@@ -168,6 +169,7 @@ typedef	struct	nat	{
 #define	NAT_DEBUG	0x800000
 
 typedef	struct	ipnat	{
+	ipfmutex_t	in_lock;
 	struct	ipnat	*in_next;		/* NAT rule list next */
 	struct	ipnat	*in_rnext;		/* rdr rule hash next */
 	struct	ipnat	**in_prnext;		/* prior rdr next ptr */
@@ -293,25 +295,6 @@ typedef	struct	natget	{
 } natget_t;
 
 
-#undef	tr_flags
-typedef	struct	nattrpnt	{
-	struct	in_addr	tr_dstip;	/* real destination IP# */
-	struct	in_addr	tr_srcip;	/* real source IP# */
-	struct	in_addr	tr_locip;	/* local source IP# */
-	u_int	tr_flags;
-	int	tr_expire;
-	u_short	tr_dstport;	/* real destination port# */
-	u_short	tr_srcport;	/* real source port# */
-	u_short	tr_locport;	/* local source port# */
-	struct	nattrpnt	*tr_hnext;
-	struct	nattrpnt	**tr_phnext;
-	struct	nattrpnt	*tr_next;
-	struct	nattrpnt	**tr_pnext;	/* previous next */
-} nattrpnt_t;
-
-#define	TN_CMPSIZ	offsetof(nattrpnt_t, tr_hnext)
-
-
 /*
  * This structure gets used to help NAT sessions keep the same NAT rule (and
  * thus translation for IP address) when:
@@ -319,6 +302,8 @@ typedef	struct	nattrpnt	{
  * (b) different IP add
  */
 typedef	struct	hostmap	{
+	struct	hostmap	*hm_hnext;
+	struct	hostmap	**hm_phnext;
 	struct	hostmap	*hm_next;
 	struct	hostmap	**hm_pnext;
 	struct	ipnat	*hm_ipnat;
@@ -370,7 +355,7 @@ typedef	struct	natstat	{
 	u_int	ns_trpntab_sz;
 	u_int	ns_hostmap_sz;
 	nat_t	*ns_instances;
-	nattrpnt_t *ns_trpntlist;
+	hostmap_t *ns_maplist;
 	u_long	*ns_bucketlen[2];
 } natstat_t;
 
@@ -424,6 +409,7 @@ extern	u_int	ipf_hostmap_sz;
 extern	u_int	fr_nat_maxbucket;
 extern	u_int	fr_nat_maxbucket_reset;
 extern	int	fr_nat_lock;
+extern	int	fr_nat_doflush;
 extern	void	fr_natsync __P((void *));
 extern	u_long	fr_defnatage;
 extern	u_long	fr_defnaticmpage;
@@ -441,7 +427,7 @@ extern	natstat_t	nat_stats;
 #if defined(__OpenBSD__)
 extern	void	nat_ifdetach __P((void *));
 #endif
-extern	int	fr_nat_ioctl __P((caddr_t, ioctlcmd_t, int));
+extern	int	fr_nat_ioctl __P((caddr_t, ioctlcmd_t, int, int, void *));
 extern	int	fr_natinit __P((void));
 extern	nat_t	*nat_new __P((fr_info_t *, ipnat_t *, nat_t **, u_int, int));
 extern	nat_t	*nat_outlookup __P((fr_info_t *, u_int, u_int, struct in_addr,
@@ -466,9 +452,11 @@ extern	void	fr_natexpire __P((void));
 extern	void	nat_log __P((struct nat *, u_int));
 extern	void	fix_incksum __P((fr_info_t *, u_short *, u_32_t));
 extern	void	fix_outcksum __P((fr_info_t *, u_short *, u_32_t));
+extern	void	fr_ipnatderef __P((ipnat_t **));
 extern	void	fr_natderef __P((nat_t **));
 extern	u_short	*nat_proto __P((fr_info_t *, nat_t *, u_int));
 extern	void	nat_update __P((fr_info_t *, nat_t *, ipnat_t *));
 extern	void	fr_setnatqueue __P((nat_t *, int));
+extern	void	fr_hostmapdel __P((hostmap_t **));
 
 #endif /* __IP_NAT_H__ */
