@@ -37,7 +37,7 @@
  *   o The enclosed hack of STREAMS support is pretty sick and most likely
  *     broken.
  *
- *	$Id: ip_rpcb_pxy.c,v 2.25.2.3 2005/02/04 10:22:56 darrenr Exp $
+ *	$Id: ip_rpcb_pxy.c,v 2.25.2.4 2006/07/14 06:12:18 darrenr Exp $
  */
 
 #define	IPF_RPCB_PROXY
@@ -1271,7 +1271,7 @@ ippr_rpcb_getnat(fin, nat, proto, port)
 			return(-1);
 		}
 		if (fi.fin_state != NULL)
-			fr_statederef(&fi, (ipstate_t **)&fi.fin_state);
+			fr_statederef((ipstate_t **)&fi.fin_state);
 	}
 
 	return(0);
