@@ -30,8 +30,6 @@
 #include <netinet/ip_frag.h>
 #include <netinet/ip_sync.h>
 
-extern	struct	selinfo	ipfselwait[IPL_LOGSIZE];
-
 #if __FreeBSD_version >= 502116
 static struct cdev *ipf_devs[IPL_LOGSIZE];
 #else
@@ -173,7 +171,7 @@ ipf_modload()
 	char *defpass, *c, *str;
 	int i, j, error;
 
-	error = iplattach();
+	error = ipfattach();
 	if (error)
 		return error;
 
@@ -225,7 +223,7 @@ ipf_modunload()
 		return EBUSY;
 
 	if (fr_running >= 0) {
-		error = ipldetach();
+		error = ipfdetach();
 		if (error != 0)
 			return error;
 	} else
